@@ -1,27 +1,24 @@
 # SPDX-FileCopyrightText: (c) 2021 Artёm IG <github.com/rtmigo>
 # SPDX-License-Identifier: MIT
-import io
-import shutil
+import os
 import sys
 from pathlib import Path
-from tempfile import TemporaryDirectory
 from subprocess import run as run_process, CalledProcessError, PIPE, STDOUT, \
     CompletedProcess
-from typing import Optional, List, Union, Iterator, Type
-import venv
-import os
+from tempfile import TemporaryDirectory
+from typing import Optional, List, Union, Type
 
+from ._cleaner import BuildCleaner
 from ._exceptions import TwineCheckFailed, FailedToInstallPackage, \
     CannotInitializeEnvironment, CodeExecutionFailed
+from ._venvs import TempVenv
 
 
-def print_command(cmd, title, at):
+def print_command(cmd: Union[str, List[str]], title: str, at: str):
     print()
-
     print('== ' + title.upper() + ' ' +
           ('=' * (80 - len(at) - len(title) - 4 - 4)) +
           ' ' + at + ' ==')
-
     if isinstance(cmd, str):
         print(cmd)
     else:
@@ -30,60 +27,6 @@ def print_command(cmd, title, at):
     print()
 
 
-def _venv_dir_to_executable(venv_dir: str) -> str:
-    p = os.path.join(venv_dir, 'bin', 'python')
-    if os.path.exists(p):
-        return p
-    p = os.path.join(venv_dir, 'Scripts', 'python.exe')
-    if os.path.exists(p):
-        return p
-    raise FileNotFoundError(
-        f"Cannot find Python executable inside {venv_dir}")
-
-
-class TempVenv:
-    """Creates virtual environment in a temporary directory.
-    Removes the directory and the environment when `cleanup` is called.
-    Can be used as context manager:
-
-    with TempVenv() as python_exe:
-        run([python_exe, '-m', 'module'])
-    """
-
-    def __init__(self):
-        self._temp_dir: Optional[TemporaryDirectory] = None
-        self._executable = None
-
-    @property
-    def executable(self):
-        if self._executable is None:
-            self._executable = _venv_dir_to_executable(self.venv_dir)
-            print(f"The python executable: {self._executable}")
-        return self._executable
-
-    @property
-    def venv_dir(self) -> str:
-        return self._temp_dir.name
-
-    def create(self):
-        self._temp_dir = TemporaryDirectory()
-        assert os.path.exists(self.venv_dir)
-        assert os.path.isdir(self.venv_dir)
-        print(f"Initializing venv in {self.venv_dir}")
-        venv.create(env_dir=self.venv_dir, with_pip=True, clear=True)
-
-    def cleanup(self):
-        print(f"Removing temp venv dir {self._temp_dir.name}")
-        self._temp_dir.cleanup()
-
-    def __enter__(self) -> str:
-        self.create()
-        return self.executable
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self.cleanup()
-
-
 def find_latest_wheel(parent_dir: Path) -> Path:
     """Finds *.whl file with the latest modification time"""
     files = [(f.stat().st_mtime, f) for f in parent_dir.glob('*.whl')]
@@ -95,27 +38,42 @@ def find_latest_wheel(parent_dir: Path) -> Path:
 
 
 class Runner:
-    """Runs commands with the same executable file (until reformat_args=False).
-    Prints the commands to the stdout with the same 'at' comments."""
+    """Prints the commands to the stdout with the same 'at' comments.
+    Runs python commands with the same executable.
+    """
 
-    def __init__(self, exe, at):
-        self.exe = exe
+    def __init__(self, python_exe, at):
+        self.python_exe = python_exe
         self.at = at
 
-    def run(self,
-            args: Union[str, List[str]],
-            title: str,
-            reformat_args: bool = True,
-            cwd: Union[Path, str] = None,
-            exception: Type[BaseException] = None,
-            executable: str = None,
-            shell: bool = False):
+    def python(self,
+               args: Union[str, List[str]],
+               title: str,
+               cwd: Union[Path, str] = None,
+               exception: Type[BaseException] = None):
 
-        if reformat_args:
-            args_list = args.split() if isinstance(args, str) else args
-            args_list = [self.exe] + args_list
-        else:
-            args_list = args
+        args_list = args.split() if isinstance(args, str) else args
+        args_list = [self.python_exe] + args_list
+        return self._run(args_list, title, cwd, exception)
+
+    def command(self,
+                args: Union[str, List[str]],
+                title: str,
+                cwd: Union[Path, str] = None,
+                exception: Type[BaseException] = None,
+                executable: str = None,
+                shell: bool = False):
+        args_list = args
+        return self._run(args_list, title, cwd, exception,
+                         executable=executable, shell=shell)
+
+    def _run(self,
+             args_list,
+             title: str,
+             cwd: Union[Path, str] = None,
+             exception: Type[BaseException] = None,
+             executable: str = None,
+             shell: bool = False):
 
         print_command(cmd=args_list, at=self.at, title=title)
 
@@ -140,59 +98,6 @@ class Runner:
         return cp
 
 
-class BuildCleaner:
-    """Removes "dist", "build" and "*.egg-info" directories if they did not
-    exist when the object was created."""
-
-    def __init__(self, project_dir: Path):
-        self.project_dir = project_dir.absolute()
-        self._build_existed = self.build_dir.exists()
-        self._distr_existed = self.distr_dir.exists()
-        self._eggs_existed = list(self.egg_infos)
-
-    @property
-    def build_dir(self) -> Path:
-        return self.project_dir / "build"
-
-    @property
-    def distr_dir(self) -> Path:
-        return self.project_dir / "distr"
-
-    @property
-    def egg_infos(self) -> Iterator[Path]:
-        return self.project_dir.glob('*.egg-info')
-
-    def cleanup(self):
-
-        def remove(p: Path):
-            shutil.rmtree(p, ignore_errors=True)
-
-        if not self._build_existed:
-            remove(self.build_dir)
-        if not self._distr_existed:
-            remove(self.distr_dir)
-
-        for egg in self.egg_infos:
-            if egg not in self._eggs_existed:
-                remove(egg)
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self.cleanup()
-
-
-def run_as_bash_script(script: str, input: bytes = None) -> CompletedProcess:
-    """Runs the provided string as a .sh script."""
-    # almost same as in VIEN
-
-    # we need executable='/bin/bash' for Ubuntu 18.04, it will run
-    # '/bin/sh' otherwise. For MacOS 10.13 it seems to be optional
-    return run_process(script, shell=True, executable='/bin/bash',
-                       input=input)
-
-
 class Package:
     """During initialization, this object creates a .whl distribution
     and installs the packages from the distribution into a test virtual
@@ -221,18 +126,18 @@ class Package:
 
         # INSTALLING BUILD ####################################################
 
-        builder.run('-m pip install --upgrade pip',
-                    title='Upgrading pip',
-                    exception=CannotInitializeEnvironment)
-        builder.run('-m pip install --upgrade build',
-                    title='Installing build',
-                    exception=CannotInitializeEnvironment)
+        builder.python('-m pip install --upgrade pip',
+                       title='Upgrading pip',
+                       exception=CannotInitializeEnvironment)
+        builder.python('-m pip install --upgrade build',
+                       title='Installing build',
+                       exception=CannotInitializeEnvironment)
 
         with TemporaryDirectory() as temp_dist_dir:
             # BUILDING ########################################################
 
             with BuildCleaner(self.project_source_dir):
-                builder.run(
+                builder.python(
                     ['-m', 'build', '--outdir', temp_dist_dir, '--wheel'],
                     title='Building the .whl',
                     cwd=self.project_source_dir)
@@ -244,15 +149,15 @@ class Package:
 
             # TWINE CHECK #####################################################
 
-            builder.run('-m pip install --upgrade twine',
-                        title='Installing twine',
-                        exception=CannotInitializeEnvironment)
+            builder.python('-m pip install --upgrade twine',
+                           title='Installing twine',
+                           exception=CannotInitializeEnvironment)
 
             # running twine checks on the new file
-            builder.run(['-m', 'twine', 'check',
-                         os.path.join(temp_dist_dir, '*'), '--strict'],
-                        title='Twine check',
-                        exception=TwineCheckFailed)
+            builder.python(['-m', 'twine', 'check',
+                            os.path.join(temp_dist_dir, '*'), '--strict'],
+                           title='Twine check',
+                           exception=TwineCheckFailed)
 
             # TEST VENV #######################################################
 
@@ -262,11 +167,11 @@ class Package:
             installer_python = self.installer_venv.__enter__()
             self._installer = Runner(installer_python, at='installer venv')
 
-            self._installer.run('-m pip install --upgrade pip',
-                                title='Upgrading pip',
-                                exception=CannotInitializeEnvironment)
+            self._installer.python('-m pip install --upgrade pip',
+                                   title='Upgrading pip',
+                                   exception=CannotInitializeEnvironment)
 
-            self._installer.run(
+            self._installer.python(
                 ['-m', 'pip', 'install', '--force-reinstall', str(whl)],
                 title=f'Installing {whl.name}',
                 exception=FailedToInstallPackage)
@@ -285,12 +190,13 @@ class Package:
             o = o.rstrip()
         return o
 
-    def run_python_code(self, code: str, rstrip: bool = True):
+    def run_python_code(self, code: str, rstrip: bool = True) -> str:
         with TemporaryDirectory() as temp_current_dir:
-            cp = self._installer.run(['-c', code],
-                                     title="Running Python code (cwd is temp dir)",
-                                     cwd=temp_current_dir,
-                                     exception=CodeExecutionFailed)
+            cp = self._installer.python(
+                ['-c', code],
+                title="Running Python code (cwd is temp dir)",
+                cwd=temp_current_dir,
+                exception=CodeExecutionFailed)
 
             return self._output(cp, rstrip)
 
@@ -299,10 +205,9 @@ class Package:
         return os.path.exists("/bin/bash")
 
     def _run_bash_code(self, code: str, rstrip: bool = True):
-        with TemporaryDirectory() as temp_current_dir:
-            activate = os.path.join(self.installer_venv.venv_dir, 'bin',
-                                    'activate')
 
+        with TemporaryDirectory() as temp_cwd:
+            activate = self.installer_venv.paths.posix_bash_activate
             code = '\n'.join(["#!/bin/bash",
                               "set -e",
                               f'source "{activate}"',
@@ -310,50 +215,40 @@ class Package:
 
             # we need executable='/bin/bash' for Ubuntu 18.04, it will run
             # '/bin/sh' otherwise. For MacOS 10.13 it seems to be optional
-            cp = self._installer.run(
+            cp = self._installer.command(
                 code,
-                reformat_args=False,
                 title="Running Bash code (cwd is temp dir)",
-                cwd=temp_current_dir,
-                shell=True, executable='/bin/bash',
-                # input=code,
+                cwd=temp_cwd,
+                executable='/bin/bash',
+                shell=True,
                 exception=CodeExecutionFailed)
 
             return self._output(cp, rstrip)
 
     def _run_cmdexe_code(self, code: str, rstrip: bool = True):
         """Runs command in cmd.exe"""
-        with TemporaryDirectory() as temp_current_dir:
-            # file that activates the venv
-            activate_bat = os.path.join(
-                self.installer_venv.venv_dir,
-                'Scripts',
-                'activate.bat')
-            assert os.path.exists(activate_bat), "activate.bat not found"
+        with TemporaryDirectory() as temp_cwd:
+            activate_bat = self.installer_venv.paths.windows_cmdexe_activate
 
             # temp file with commands to run
-            temp_bat_file = Path(temp_current_dir) / "_run_cmdexe_code.bat"
+            temp_bat_file = Path(temp_cwd) / "_run_cmdexe_code.bat"
             temp_bat_file.write_text(
                 '\n'.join([f"CALL {activate_bat}",
                            code]))
 
             # todo param /u formats output as unicode?
-            cp = self._installer.run(
+            cp = self._installer.command(
                 ["cmd.exe", "/q", "/c", str(temp_bat_file)],
-                reformat_args=False,
                 title="Running code in cmd.exe (cwd is temp dir)",
-                cwd=temp_current_dir,
-                shell=True,  # executable='/bin/bash',
-                # input=code,
+                cwd=temp_cwd,
+                shell=False,
                 exception=CodeExecutionFailed)
 
             return self._output(cp, rstrip)
 
     def run_shell_code(self, code: str, rstrip: bool = True) -> str:
-
         if os.name == 'nt':
             method = self._run_cmdexe_code
         else:
             method = self._run_bash_code
-
         return method(code, rstrip=rstrip)
